use std::fs;
use std::error::Error;
use serde::{Serialize, Deserialize};

#[derive(Serialize, Deserialize, Debug)]
enum Social {
    Twitter(String),
    ICQ(u64),
    Nothing,
}

#[derive(Serialize, Deserialize, Debug)]
struct Contact {
    name: String,
    phone: u32,
    social: Social,
}

type Contacts = Vec<Contact>;

fn build_contacts() -> Contacts {
    vec![
        Contact { name: "John".to_string(), phone: 12345, social: Social::Twitter("@the_john".to_string()) },
        Contact { name: "Mary".to_string(), phone: 9876543, social: Social::ICQ(111234) },
        Contact { name: "Jane".to_string(), phone: 555555, social: Social::Nothing },
    ]
}

fn create_contacts() -> Result<(), Box<dyn Error>> {
    let contacts = build_contacts();
    let serialized = serde_json::to_string(&contacts)?;
    println!("serialized: {}", serialized);
    let mut f = fs::File::create("/tmp/contacts.json")?;
    serde_json::to_writer_pretty(&mut f, &contacts)?;
    Ok(())
}

fn main() -> Result<(), Box<dyn Error>> {
    create_contacts()?;
    Ok(())
}
